#!/usr/bin/python3

__all__ = ['VideoEngines', 'Radar', ]
__version__ = '0.1.1'
__author__ = 'David Johnston'

import os
import sys
from enum import Enum
import ctypes

try:
    from roboradar import config
    from roboradar.fields import fields, fieldFiles, fieldNames, fieldThemes
    import roboradar.robots as robots
except ImportError:
    import config
    from fields import fields, fieldFiles, fieldNames, fieldThemes
    import robots

config.load_config()
conf = config.get_config()

robotList = robots.getRobots()

# Force program to run as a package/module. Recommended for compatibility.
# This is designed to run as a module, but this can cause issues when
# developing in IDLE. Non-module mode is of low priority, and may not work AT
# ALL!!!
# Default: True
# FORCE_RUN_AS_MODULE = True

if (conf["SYSTEM"]["FORCE_RUN_AS_MODULE"]) and __package__ is None:
    print("""Not running as module, restarting. Please run using 'py -m
RoboRadar.__init__'""")
    existingcwd = os.getcwd()
    os.chdir(os.path.dirname(os.path.realpath(__file__))[:-10])
    os.system("py -m RoboRadar.__init__")
    os.chdir(existingcwd)
    exit()


'''
============================

- DEFINITIONS -

============================

These are standard definitions for the program to use.
Do not mess with these unless you know what you are doing!
'''


class VideoEngines(Enum):
    native = "native"
    numpy = "numpy"
    pygame = "pygame"


VERSION = __version__

'''
============================

- CONFIG -

============================

Default configuration stored alongside the program.
These are the settings that will be loaded into an .ini file the
first time RoboRadar is run.
'''

# Number of FPS to run the screen at. Recommended 30, 60, or the refresh rate
# of the monitor
# Default: 60
# FPS = 60

# Team number of FRC Team
# Default: 0
# TEAM_NUMBER = 0

# NetworkTables server address, leave this unchanged to use the team number.
# Each {}{} will be filled with part of the team number
# Default: "10.{}.{}.2"
# SERVER_ADDRESS = "10.{}.{}.2"

# Set the starting screen width and height
# Default: 480, 640
# INITIAL_SCREEN_WIDTH, INITIAL_SCREEN_HEIGHT = 480, 640

# Enable/disable antialiased polygons and lines
# Default: True
# ANTIALIASING = True

# Enable/disable filled polygons (gives everything a wireframe look. NOT
# recommended)
# Default: True
# FILLED_POLYGONS = True

# Set video engine for graphics
# Default: pygame
# VIDEO_ENGINE = VideoEngines.pygame

# Set starting field
# Default: "FRC_2020"
# DEFAULT_FIELD = "FRC_2020"

# Dont mess with this. It WILL cause syncing issues
'''CONFIG = {
    "FPS": FPS,
    "TEAM_NUMBER": TEAM_NUMBER,
    "SERVER_ADDRESS": SERVER_ADDRESS,
    "INITIAL_SCREEN_WIDTH": INITIAL_SCREEN_WIDTH,
    "INITIAL_SCREEN_HEIGHT": INITIAL_SCREEN_HEIGHT,
    "ANTIALIASING": ANTIALIASING,
    "FILLED_POLYGONS": FILLED_POLYGONS,
    "VIDEO_ENGINE": VIDEO_ENGINE,
    "DEFAULT_FIELD": DEFAULT_FIELD
    }
'''

if VideoEngines[conf["VIDEO"]["ENGINE"]] is VideoEngines.pygame:
    os.environ['PYGAME_HIDE_SUPPORT_PROMPT'] = "hide"
    import pygame
    import pygame.gfxdraw
    import pygame.locals
else:
    raise ValueError

_pgFlag = pygame.RESIZABLE | pygame.HWSURFACE | pygame.DOUBLEBUF


def start_independent(flags=_pgFlag):
    '''Start an independent RoboRadar WindowsError
flags: flags for pygame.display.set_mode'''
    appid = 'ShortSirkit.RoboRadar.RoboRadar.1_0_0'
    ctypes.windll.shell32.SetCurrentProcessExplicitAppUserModelID(appid)

    pygame.init()

    screen = pygame.display.set_mode(
        conf["VIDEO"]["SCREEN_DIMENSIONS"],
        pygame.RESIZABLE | flags)
    pygame.display.set_caption("RoboRadar v{} - Team {}".format(
        VERSION,
        conf["TEAM"]["NUMBER"])
        )
    pygame.display.set_icon(pygame.image.load(__file__[:-11] + "icon.png"))

    clock = pygame.time.Clock()

    r = Radar(conf["VIDEO"]["SCREEN_DIMENSIONS"])
    r.loadField(conf["FIELD"]["NAME"])
    bb = robotList["BoxBot"]()
    r.add_ds(bb)

    while True:
        screen.fill((249, 249, 249))

        for event in pygame.event.get():
            if event.type == pygame.locals.QUIT:
                pygame.quit()
                sys.exit()
            if event.type == pygame.locals.VIDEORESIZE:
                r.resize(event.size)
                screen = pygame.display.set_mode(
                    (event.w, event.h),
                    pygame.RESIZABLE
                    )

        # Update.

        screen.blit(r.pygame_render(), (0, 0))

        # Draw.
        pygame.display.flip()
        clock.tick(conf["VIDEO"]["FPS"])


class Radar:
    _dsArray = []

    def __init__(
                 self, dimensions,
                 interface=conf["VIDEO"]["ENGINE"],
                 *args, **kwargs):
        self.dimensions = dimensions
        self.fieldIndex = None
        self.field = None
        if VideoEngines[interface] is VideoEngines.pygame:
            self._init_pygame(*args, **kwargs)

    def loadField(self, search, *args, **kwargs):
        if isinstance(search, int) and 0 <= int(search) < len(fields):
            self.fieldIndex = search
        else:
            self.fieldIndex = fieldFiles.get(search, None)
        if self.fieldIndex is None:
            self.fieldIndex = fieldNames.get(search, None)
        if self.fieldIndex is None:
            self.fieldIndex = fieldThemes.get(search, None)
        if self.fieldIndex is None:
            raise ValueError
        self.field = fields[self.fieldIndex]
        self._loadField_engineSpecific()

    def resize(self, dimensions):
        self.dimensions = dimensions
        self._resize_engineSpecific()

    def add_ds(self, ds):
        self._dsArray.append(ds)

    def _init_pygame(self, *args, **kwargs):
        self._loadField_engineSpecific = self._loadField_pygame
        self._visibleSurface = pygame.Surface(self.dimensions)
        self._resize_engineSpecific = self._resize_pygame

    def _loadField_pygame(self):
        self._resize_pygame()

    def _resize_pygame(self):
        fd = self.field.Data
        dimen = self.dimensions
        self._visibleSurface = pygame.Surface(dimen)
        self._visibleSurface.fill((0, 0, 0))
        if fd["width"] / fd["height"] <= dimen[0] / dimen[1]:
            height = dimen[1]
            width = fd["width"] / fd["height"] * height
        else:
            width = dimen[0]
            height = fd["height"] / fd["width"] * width
        self._offset = (
            int((dimen[0] - width) / 2),
            int((dimen[1] - height) / 2)
            )
        self._staticHeight, self._staticWidth = int(height), int(width)
        self._staticSurface = pygame.Surface(
            (self._staticWidth,  self._staticHeight)
            )
        for shape in fd["static-shapes"]:
            self._pygame_draw(shape, self._staticSurface)

    def _pygame_convertCoordinateSpace(self, points, offset=(0, 0)):
        p = []
        center = self.field.Data["center"]
        dimen = (self.field.Data["width"], self.field.Data["height"])
        for point in points:
            x = (point[0] + center[0]) / dimen[0] * self._staticWidth
            y = (-point[1] + center[1]) / dimen[1] * self._staticHeight
            p.append((int(x + offset[0]), int(y + offset[1])))
        return p

    def _pygame_draw(self, shape, surface, offset=(0, 0)):
        p = self._pygame_convertCoordinateSpace(shape["points"], offset)
        if shape["type"] == "polygon":
            if "filled" in shape["style"]:
                pygame.gfxdraw.filled_polygon(
                    surface,
                    p,
                    shape["color"]
                    )
            if "outline" in shape["style"]:
                pygame.gfxdraw.polygon(
                    surface,
                    p,
                    shape["color"]
                    )
            if "aa" in shape["style"]:
                pygame.gfxdraw.aapolygon(
                    surface,
                    p,
                    shape["color"]
                    )
        elif shape["type"] == "line":
            if "outline" in shape["style"]:
                pygame.gfxdraw.line(
                    surface,
                    p[0][0],
                    p[0][1],
                    p[1][0],
                    p[1][1],
                    shape["color"]
                    )
            if "aa" in shape["style"]:
                pygame.gfxdraw.aapolygon(
                    surface,
                    p,
                    shape["color"]
                    )

    def pygame_render(self):
        self._visibleSurface.fill((0, 0, 0))
        self._visibleSurface.blit(self._staticSurface, self._offset)
        for ds in self._dsArray:
            for shape in ds.draw(self.field.Data["orientation"]):
                self._pygame_draw(shape, self._visibleSurface, self._offset)
        return self._visibleSurface


if __name__ == "__main__":
    if conf["TEAM"]["NUMBER"] == 0:
        num = input("TEAM.NUMBER option is set to 0, please enter one: ")
        try:
            conf["TEAM"]["NUMBER"] = float(num.strip())
        except ValueError:
            print("No TEAM.NUMBER entered, or non-numeric input given.")
            exit(1)
    if conf["TEAM"]["NUMBER"] <= 0:
        print("Invalid TEAM.NUMBER. TEAM.NUMBER must be greater than 0.")
        exit(1)
    if int(conf["TEAM"]["NUMBER"]) != conf["TEAM"]["NUMBER"]:
        print("Invalid TEAM.NUMBER. TEAM.NUMBER must be an integer.")
        exit(1)
    if not isinstance(conf["TEAM"]["NUMBER"], int):
        conf["TEAM"]["NUMBER"] = int(conf["TEAM"]["NUMBER"])
    if len(str(conf["TEAM"]["NUMBER"])) > 4:
        print("Invalid TEAM.NUMBER. TEAM.NUMBER must be 4 digits or less.")
        exit(1)
    config.set_nt_address(conf["TEAM"]["NUMBER"])
    conf = config.get_config()
    start_independent()



#!/usr/bin/python3

import importlib
import os
from abc import abstractmethod


try:
    from roboradar import config
    from roboradar import dynamic_shape
except ImportError:
    import config
    import dynamic_shape
conf = config.get_config()


class Robot(dynamic_shape.DynamicShape):

    @classmethod
    @abstractmethod
    def getInfo(self):
        '''This method is called to get info associated with the robot
see the docs (to be made) for more info'''
        pass


def getRobots():
    robots = {}
    for f in os.listdir(__file__[:-11]):
        if f.endswith(".py") and f != "__init__.py":
            if __package__:
                module = "." + f[:-3]
            else:
                module = f[:-3]
            mod = importlib.import_module(
                module,
                package=__package__
                )
            robots.update(mod.types)
    return robots



from math import pi

fieldHeight = (52 * 12 + 5.25) / 2
edgeHeight = fieldHeight + (10 * 12 + 9 + 1/8)
fieldWidth = (26 * 12 + 11.25) / 2
edgeWidth = (30 * 12) / 2

Data = {
    "name": "FRC 2020 Field",
    "theme": "Infinite Recharge",
    "file": "FRC_2020",
    "version": "1.0.0",
    "author": "David Johnston",
    "format": "PSF1",  # Python Script Field - Version 1
    "aliases": {
        "FRC 2020 FIELD 1.0.0",
        "FRC 2020 FIELD 1.0",
        "FRC 2020 FIELD 1",
        "FRC 2020 FIELD V1.0.0",
        "FRC 2020 FIELD V1.0",
        "FRC 2020 FIELD V1",
        "FRC_2020_1.0.0",
        "FRC_2020_1.0",
        "FRC_2020_1",
        "FRC_2020_V1.0.0",
        "FRC_2020_V1.0",
        "FRC_2020_V1",
        "FRC_2020V1.0.0",
        "FRC_2020V1.0",
        "FRC_2020V1",
        },
    "date": "2020-02-08T16:19-05:00",  # -05:00 Eastern no DST. DST is -04:00
    "units": "inches",
    "width": edgeWidth * 2,
    "height": edgeHeight * 2,
    "center": (edgeWidth, edgeHeight),
    "orientation": 0,
    "static-shapes": [
        {
            "name": "carpet",
            "type": "polygon",
            "style": ("filled"),
            "color": (43, 43, 43),
            "layer": 0,
            "points": [
                (edgeWidth, edgeHeight),
                (edgeWidth, -edgeHeight),
                (-edgeWidth, -edgeHeight),
                (-edgeWidth, edgeHeight)
                ]
            },
        {
            "name": "red alliance station",
            "type": "polygon",
            "style": ("filled", "aa"),
            "color": (96, 32, 32),
            "layer": 0,
            "points": [
                (edgeWidth, -edgeHeight),
                (edgeWidth, -edgeHeight+12*12+10+7/8),
                (fieldWidth, -edgeHeight+12*12+10+7/8),
                (fieldWidth-94.66+24, -edgeHeight+10*12+9+1/8),
                (-fieldWidth+94.66-24, -edgeHeight+10*12+9+1/8),
                (-fieldWidth, -edgeHeight+12*12+10+7/8),
                (-edgeWidth, -edgeHeight+12*12+10+7/8),
                (-edgeWidth, -edgeHeight)
                ]
            },
        {
            "name": "blue alliance station",
            "type": "polygon",
            "style": ("filled", "aa"),
            "color": (32, 32, 96),
            "layer": 0,
            "points": [
                (edgeWidth, edgeHeight),
                (edgeWidth, edgeHeight-12*12-10-7/8),
                (fieldWidth, edgeHeight-12*12-10-7/8),
                (fieldWidth-94.66+24, edgeHeight-10*12-9-1/8),
                (-fieldWidth+94.66-24, edgeHeight-10*12-9-1/8),
                (-fieldWidth, edgeHeight-12*12-10-7/8),
                (-edgeWidth, edgeHeight-12*12-10-7/8),
                (-edgeWidth, edgeHeight)
                ]
            },
        {
            "name": "field",
            "type": "polygon",
            "style": ("outline", "aa"),
            "color": (128, 128, 128),
            "layer": 0,
            "points": [

                (fieldWidth, -edgeHeight+12*12+10+7/8),
                (fieldWidth-94.66+24, -edgeHeight+10*12+9+1/8),
                (-fieldWidth+94.66-24, -edgeHeight+10*12+9+1/8),
                (-fieldWidth, -edgeHeight+12*12+10+7/8),
                (-fieldWidth, edgeHeight-12*12-10-7/8),
                (-fieldWidth+94.66-24, edgeHeight-10*12-9-1/8),
                (fieldWidth-94.66+24, edgeHeight-10*12-9-1/8),
                (fieldWidth, edgeHeight-12*12-10-7/8),
                ]
            },
        {
            "name": "red initiation line",
            "type": "polygon",
            "style": ("filled", "aa"),
            "color": (255, 255, 255),
            "layer": 0,
            "points": [
                (fieldWidth, fieldHeight-10*12),
                (fieldWidth, fieldHeight-10*12-2),
                (-fieldWidth, fieldHeight-10*12-2),
                (-fieldWidth, fieldHeight-10*12)
                ]
            },
        {
            "name": "blue initiation line",
            "type": "polygon",
            "style": ("filled", "aa"),
            "color": (255, 255, 255),
            "layer": 0,
            "points": [
                ((26 * 12 + 11.25) / 2, -194.625),
                ((26 * 12 + 11.25) / 2, -194.625+2),
                (-(26 * 12 + 11.25) / 2, -194.625+2),
                (-(26 * 12 + 11.25) / 2, -194.625)
                ]
            }
        ]
    }

if __name__ == "__main__":
    print(Data)



import importlib, os
fields = []
fieldFiles = {}
fieldNames = {}
fieldThemes = {}
for file in os.listdir(__file__[:-11]):
    if file.endswith(".py") and file != "__init__.py":
        if __package__:
            module = "." + file[:-3]
        else:
            module = file[:-3]
        fieldIndex = len(fields)
        fields.append(importlib.import_module(module, package=__package__))
        fieldFiles[fields[-1].Data["file"]] = fieldIndex #TODO: add version overriding
        fieldNames[fields[-1].Data["name"]] = fieldIndex
        fieldThemes[fields[-1].Data["theme"]] = fieldIndex



from abc import ABC, abstractmethod
import math


class DynamicShape(ABC):
    x = 0
    y = 0
    r = 0
    d = None
    rsin = None
    rcos = None
    rtan = None

    @abstractmethod
    def getShapes(self):
        '''This method is called once per frame.
it should return a list of shapes to be drawn on the screen.'''
        pass

    def draw(self, r_adjust=0):
        shapes = self.getShapes()
        if self.d is None:
            r = self.r
        else:
            r = math.radians(self.d)
        if r_adjust == 0:
            if self.rsin is None:
                rsin = math.sin(r)
            else:
                rsin = self.rsin
            if self.rcos is None:
                rcos = math.cos(r)
            else:
                rcos = self.rcos
            # This next section is commented out because it may be needed in
            # the future, but for now isn't needed. Therefore if I don't need
            # to do the calculations, I won't waste the cycles.
            '''if self.rtan is None:
                rtan = math.tan(r)
            else:
                rtan = self.rtan'''
        else:
            r = r + r_adjust
            rsin = math.sin(r)
            rcos = math.cos(r)
        for shape in shapes:
            if shape["type"] == "polygon" or shape["type"] == "line":
                if shape["coordinate-space"] == "local":
                    points = []
                    for point in shape["points"]:
                        x = point[0] * rcos - point[1] * rsin + self.x
                        y = point[0] * rsin + point[1] * rcos + self.y
                        points.append((x, y))
                    shape["points"] = points
                    yield shape



#!/usr/bin/python3

import logging
from networktables import NetworkTables
from . import Robot, config

conf = config.get_config()

logging.basicConfig(level=logging.DEBUG)


class NetworkTableBot(Robot):

    def __init__(self, *args, **kwargs):
        NetworkTables.initialize(server=kwargs.get(
            "server",
            conf["ROBOT"]["IP_ADDRESS"]
            ))
        self.nt = NetworkTables.getTable("RoboRadar")

    @property
    def x(self):
        return self.nt.getNumber("posX", 0)

    @property
    def y(self):
        return self.nt.getNumber("posY", 0)

    @property
    def r(self):
        return self.nt.getNumber("posR", 0)

    '''@property
    def rsin(self):
        return self.nt.getNumber("posRSin", 1)

    @property
    def rcos(self):
        return self.nt.getNumber("posRCos", 0)

    @property
    def rtan(self):
        return self.nt.getNumber("posRTan", 0)'''

    def getTeamColor(self):
        # return purple if no color defined
        return tuple(self.nt.getNumberArray("color", (255, 0, 255)))


class BoxBot(NetworkTableBot):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

    @property
    def w(self):
        return self.nt.getNumber("boxbotW", 27)

    @property
    def h(self):
        return self.nt.getNumber("boxbotH", 32)

    @classmethod
    def getInfo(cls):
        data = {
            "name": "BoxBot",
            "units": "inches",
            }
        return data

    def getShapes(self):
        w = self.w / 2
        h = self.h / 2
        shapes = [
            {
                 "name": "bumpers",
                 "type": "polygon",
                 "style": ("filled", "aa"),
                 "color": self.getTeamColor(),
                 "layer": 0,
                 "coordinate-space": "local",
                 "points": [
                     (-(w+3.25), (h+3.25)),
                     ((w+3.25), (h+3.25)),
                     ((w+3.25), -(h+3.25)),
                     (-(w+3.25), -(h+3.25))
                     ]
                 },
            {
                "name": "frame",
                "type": "polygon",
                "style": ("filled", "aa"),
                "color": (128, 128, 128),
                "layer": 0,
                "coordinate-space": "local",
                "points": [
                    (-w, h),
                    (w, h),
                    (w, -h),
                    (-w, -h)
                    ]
                },
            {
                "name": "pointer",
                "type": "line",
                "style": ("outline"),
                "color": (0, 0, 0),
                "layer": 0,
                "coordinate-space": "local",
                "points": [
                    (0, 0),
                    (0, h),
                    ]
                },
            {
                "name": "arrow",
                "type": "line",
                "style": ("outline"),
                "color": (0, 255, 0),
                "layer": 0,
                "coordinate-space": "local",
                "points": [
                    (0, 0),
                    (0, (h+8)),
                    ]
                }
            ]
        return shapes


# print(BoxBot())


types = {
    "BoxBot": BoxBot
    }



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()


setuptools.setup(
    name="roboradar",
    version="0.1.1",
    author="David Johnston",
    author_email="dwjgame11@gmail.com",
    description="Robot position display software for FRC",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Short-SirKit-6527/RoboRadar",
    packages=[
        "roboradar"
        ],
    package_data={
        "roboradar": [
            "icon.png",
            "RoboRadarConfig.json"
            ]
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
)


print("done")



import json
import os


_conf = {}


def find_config():
    if os.path.exists(os.getcwd() + os.sep + "RoboRadarConfig.json"):
        return os.getcwd() + os.sep + "RoboRadarConfig.json"
    return __file__[:-10] + os.sep + "RoboRadarConfig.json"


def load_config(filePath=None):
    global _conf
    if filePath is None:
        filePath = find_config()
    print(filePath)
    print(find_config())
    with open(filePath) as f:
        _conf = json.load(f)
        if _conf["ROBOT"]["IP_ADDRESS"] is None:
            _conf["ROBOT"]["IP_ADDRESS_SET"] = False
            if _conf["TEAM"]["NUMBER"] != 0:
                set_nt_address(_conf["TEAM"]["NUMBER"])
        else:
            _conf["ROBOT"]["IP_ADDRESS_SET"] = True


def set_nt_address(tn):
    if not _conf["ROBOT"]["IP_ADDRESS_SET"]:
        tn = str(tn).zfill(4)
        tn = (tn[0:2], tn[2:4])
        iaf = _conf["ROBOT"]["IP_ADDRESS_FORMAT"]
        _conf["ROBOT"]["IP_ADDRESS"] = iaf.format(tn[0], tn[1])


def get_config():
    if not _conf:
        load_config()
    return _conf
